# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from .conversation_flow_component_response import ConversationFlowComponentResponse

__all__ = ["ConversationFlowComponentListResponse"]

ConversationFlowComponentListResponse: TypeAlias = List[ConversationFlowComponentResponse]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Optional
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from .._types import SequenceNotStr

__all__ = [
    "TestCreateBatchTestParams",
    "ResponseEngine",
    "ResponseEngineResponseEngineRetellLm",
    "ResponseEngineResponseEngineConversationFlow",
]


class TestCreateBatchTestParams(TypedDict, total=False):
    response_engine: Required[ResponseEngine]
    """Response engine to use for the test cases. Custom LLM is not supported."""

    test_case_definition_ids: Required[SequenceNotStr[str]]
    """Array of test case definition IDs to run"""

    reserved_concurrency: int
    """
    Number of concurrency reserved for all other calls that are not triggered by
    batch calls, such as inbound calls.
    """


class ResponseEngineResponseEngineRetellLm(TypedDict, total=False):
    llm_id: Required[str]
    """id of the Retell LLM Response Engine."""

    type: Required[Literal["retell-llm"]]
    """type of the Response Engine."""

    version: Optional[float]
    """Version of the Retell LLM Response Engine."""


class ResponseEngineResponseEngineConversationFlow(TypedDict, total=False):
    conversation_flow_id: Required[str]
    """ID of the Conversation Flow Response Engine."""

    type: Required[Literal["conversation-flow"]]
    """type of the Response Engine."""

    version: Optional[float]
    """Version of the Conversation Flow Response Engine."""


ResponseEngine: TypeAlias = Union[ResponseEngineResponseEngineRetellLm, ResponseEngineResponseEngineConversationFlow]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from retell import Retell, AsyncRetell
from tests.utils import assert_matches_type
from retell.types import (
    ChatAgentResponse,
    ChatAgentListResponse,
    ChatAgentGetVersionsResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestChatAgent:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Retell) -> None:
        chat_agent = client.chat_agent.create(
            response_engine={
                "llm_id": "llm_234sdertfsdsfsdf",
                "type": "retell-llm",
            },
        )
        assert_matches_type(ChatAgentResponse, chat_agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Retell) -> None:
        chat_agent = client.chat_agent.create(
            response_engine={
                "llm_id": "llm_234sdertfsdsfsdf",
                "type": "retell-llm",
                "version": 0,
            },
            agent_name="Jarvis",
            analysis_successful_prompt="The agent finished the task and the call was complete without being cutoff.",
            analysis_summary_prompt="Summarize the call in a few sentences.",
            auto_close_message="Thank you for chatting. The conversation has ended.",
            data_storage_setting="everything",
            end_chat_after_silence_ms=3600000,
            language="en-US",
            opt_in_signed_url=True,
            pii_config={
                "categories": ["person_name"],
                "mode": "post_call",
            },
            post_chat_analysis_data=[
                {
                    "description": "The name of the customer.",
                    "name": "customer_name",
                    "type": "string",
                    "examples": ["John Doe", "Jane Smith"],
                }
            ],
            post_chat_analysis_model="gpt-4.1-mini",
            signed_url_expiration_ms=86400000,
            webhook_timeout_ms=10000,
            webhook_url="https://webhook-url-here",
        )
        assert_matches_type(ChatAgentResponse, chat_agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Retell) -> None:
        response = client.chat_agent.with_raw_response.create(
            response_engine={
                "llm_id": "llm_234sdertfsdsfsdf",
                "type": "retell-llm",
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        chat_agent = response.parse()
        assert_matches_type(ChatAgentResponse, chat_agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Retell) -> None:
        with client.chat_agent.with_streaming_response.create(
            response_engine={
                "llm_id": "llm_234sdertfsdsfsdf",
                "type": "retell-llm",
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            chat_agent = response.parse()
            assert_matches_type(ChatAgentResponse, chat_agent, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Retell) -> None:
        chat_agent = client.chat_agent.retrieve(
            agent_id="16b980523634a6dc504898cda492e939",
        )
        assert_matches_type(ChatAgentResponse, chat_agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve_with_all_params(self, client: Retell) -> None:
        chat_agent = client.chat_agent.retrieve(
            agent_id="16b980523634a6dc504898cda492e939",
            version=1,
        )
        assert_matches_type(ChatAgentResponse, chat_agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Retell) -> None:
        response = client.chat_agent.with_raw_response.retrieve(
            agent_id="16b980523634a6dc504898cda492e939",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        chat_agent = response.parse()
        assert_matches_type(ChatAgentResponse, chat_agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Retell) -> None:
        with client.chat_agent.with_streaming_response.retrieve(
            agent_id="16b980523634a6dc504898cda492e939",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            chat_agent = response.parse()
            assert_matches_type(ChatAgentResponse, chat_agent, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Retell) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `agent_id` but received ''"):
            client.chat_agent.with_raw_response.retrieve(
                agent_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update(self, client: Retell) -> None:
        chat_agent = client.chat_agent.update(
            agent_id="16b980523634a6dc504898cda492e939",
        )
        assert_matches_type(ChatAgentResponse, chat_agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Retell) -> None:
        chat_agent = client.chat_agent.update(
            agent_id="16b980523634a6dc504898cda492e939",
            version=1,
            agent_name="Jarvis",
            analysis_successful_prompt="The agent finished the task and the call was complete without being cutoff.",
            analysis_summary_prompt="Summarize the call in a few sentences.",
            auto_close_message="Thank you for chatting. The conversation has ended.",
            data_storage_setting="everything",
            end_chat_after_silence_ms=3600000,
            language="en-US",
            opt_in_signed_url=True,
            pii_config={
                "categories": ["person_name"],
                "mode": "post_call",
            },
            post_chat_analysis_data=[
                {
                    "description": "The name of the customer.",
                    "name": "customer_name",
                    "type": "string",
                    "examples": ["John Doe", "Jane Smith"],
                }
            ],
            post_chat_analysis_model="gpt-4.1-mini",
            response_engine={
                "llm_id": "llm_234sdertfsdsfsdf",
                "type": "retell-llm",
                "version": 0,
            },
            signed_url_expiration_ms=86400000,
            webhook_timeout_ms=10000,
            webhook_url="https://webhook-url-here",
        )
        assert_matches_type(ChatAgentResponse, chat_agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Retell) -> None:
        response = client.chat_agent.with_raw_response.update(
            agent_id="16b980523634a6dc504898cda492e939",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        chat_agent = response.parse()
        assert_matches_type(ChatAgentResponse, chat_agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Retell) -> None:
        with client.chat_agent.with_streaming_response.update(
            agent_id="16b980523634a6dc504898cda492e939",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            chat_agent = response.parse()
            assert_matches_type(ChatAgentResponse, chat_agent, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Retell) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `agent_id` but received ''"):
            client.chat_agent.with_raw_response.update(
                agent_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Retell) -> None:
        chat_agent = client.chat_agent.list()
        assert_matches_type(ChatAgentListResponse, chat_agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Retell) -> None:
        chat_agent = client.chat_agent.list(
            limit=50,
            pagination_key="16b980523634a6dc504898cda492e939",
            pagination_key_version=0,
        )
        assert_matches_type(ChatAgentListResponse, chat_agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Retell) -> None:
        response = client.chat_agent.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        chat_agent = response.parse()
        assert_matches_type(ChatAgentListResponse, chat_agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Retell) -> None:
        with client.chat_agent.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            chat_agent = response.parse()
            assert_matches_type(ChatAgentListResponse, chat_agent, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: Retell) -> None:
        chat_agent = client.chat_agent.delete(
            "oBeDLoLOeuAbiuaMFXRtDOLriTJ5tSxD",
        )
        assert chat_agent is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Retell) -> None:
        response = client.chat_agent.with_raw_response.delete(
            "oBeDLoLOeuAbiuaMFXRtDOLriTJ5tSxD",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        chat_agent = response.parse()
        assert chat_agent is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Retell) -> None:
        with client.chat_agent.with_streaming_response.delete(
            "oBeDLoLOeuAbiuaMFXRtDOLriTJ5tSxD",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            chat_agent = response.parse()
            assert chat_agent is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Retell) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `agent_id` but received ''"):
            client.chat_agent.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_get_versions(self, client: Retell) -> None:
        chat_agent = client.chat_agent.get_versions(
            "16b980523634a6dc504898cda492e939",
        )
        assert_matches_type(ChatAgentGetVersionsResponse, chat_agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_get_versions(self, client: Retell) -> None:
        response = client.chat_agent.with_raw_response.get_versions(
            "16b980523634a6dc504898cda492e939",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        chat_agent = response.parse()
        assert_matches_type(ChatAgentGetVersionsResponse, chat_agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_get_versions(self, client: Retell) -> None:
        with client.chat_agent.with_streaming_response.get_versions(
            "16b980523634a6dc504898cda492e939",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            chat_agent = response.parse()
            assert_matches_type(ChatAgentGetVersionsResponse, chat_agent, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_get_versions(self, client: Retell) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `agent_id` but received ''"):
            client.chat_agent.with_raw_response.get_versions(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_publish(self, client: Retell) -> None:
        chat_agent = client.chat_agent.publish(
            "16b980523634a6dc504898cda492e939",
        )
        assert chat_agent is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_publish(self, client: Retell) -> None:
        response = client.chat_agent.with_raw_response.publish(
            "16b980523634a6dc504898cda492e939",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        chat_agent = response.parse()
        assert chat_agent is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_publish(self, client: Retell) -> None:
        with client.chat_agent.with_streaming_response.publish(
            "16b980523634a6dc504898cda492e939",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            chat_agent = response.parse()
            assert chat_agent is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_publish(self, client: Retell) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `agent_id` but received ''"):
            client.chat_agent.with_raw_response.publish(
                "",
            )


class TestAsyncChatAgent:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncRetell) -> None:
        chat_agent = await async_client.chat_agent.create(
            response_engine={
                "llm_id": "llm_234sdertfsdsfsdf",
                "type": "retell-llm",
            },
        )
        assert_matches_type(ChatAgentResponse, chat_agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncRetell) -> None:
        chat_agent = await async_client.chat_agent.create(
            response_engine={
                "llm_id": "llm_234sdertfsdsfsdf",
                "type": "retell-llm",
                "version": 0,
            },
            agent_name="Jarvis",
            analysis_successful_prompt="The agent finished the task and the call was complete without being cutoff.",
            analysis_summary_prompt="Summarize the call in a few sentences.",
            auto_close_message="Thank you for chatting. The conversation has ended.",
            data_storage_setting="everything",
            end_chat_after_silence_ms=3600000,
            language="en-US",
            opt_in_signed_url=True,
            pii_config={
                "categories": ["person_name"],
                "mode": "post_call",
            },
            post_chat_analysis_data=[
                {
                    "description": "The name of the customer.",
                    "name": "customer_name",
                    "type": "string",
                    "examples": ["John Doe", "Jane Smith"],
                }
            ],
            post_chat_analysis_model="gpt-4.1-mini",
            signed_url_expiration_ms=86400000,
            webhook_timeout_ms=10000,
            webhook_url="https://webhook-url-here",
        )
        assert_matches_type(ChatAgentResponse, chat_agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncRetell) -> None:
        response = await async_client.chat_agent.with_raw_response.create(
            response_engine={
                "llm_id": "llm_234sdertfsdsfsdf",
                "type": "retell-llm",
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        chat_agent = await response.parse()
        assert_matches_type(ChatAgentResponse, chat_agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncRetell) -> None:
        async with async_client.chat_agent.with_streaming_response.create(
            response_engine={
                "llm_id": "llm_234sdertfsdsfsdf",
                "type": "retell-llm",
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            chat_agent = await response.parse()
            assert_matches_type(ChatAgentResponse, chat_agent, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncRetell) -> None:
        chat_agent = await async_client.chat_agent.retrieve(
            agent_id="16b980523634a6dc504898cda492e939",
        )
        assert_matches_type(ChatAgentResponse, chat_agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve_with_all_params(self, async_client: AsyncRetell) -> None:
        chat_agent = await async_client.chat_agent.retrieve(
            agent_id="16b980523634a6dc504898cda492e939",
            version=1,
        )
        assert_matches_type(ChatAgentResponse, chat_agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncRetell) -> None:
        response = await async_client.chat_agent.with_raw_response.retrieve(
            agent_id="16b980523634a6dc504898cda492e939",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        chat_agent = await response.parse()
        assert_matches_type(ChatAgentResponse, chat_agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncRetell) -> None:
        async with async_client.chat_agent.with_streaming_response.retrieve(
            agent_id="16b980523634a6dc504898cda492e939",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            chat_agent = await response.parse()
            assert_matches_type(ChatAgentResponse, chat_agent, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncRetell) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `agent_id` but received ''"):
            await async_client.chat_agent.with_raw_response.retrieve(
                agent_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncRetell) -> None:
        chat_agent = await async_client.chat_agent.update(
            agent_id="16b980523634a6dc504898cda492e939",
        )
        assert_matches_type(ChatAgentResponse, chat_agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncRetell) -> None:
        chat_agent = await async_client.chat_agent.update(
            agent_id="16b980523634a6dc504898cda492e939",
            version=1,
            agent_name="Jarvis",
            analysis_successful_prompt="The agent finished the task and the call was complete without being cutoff.",
            analysis_summary_prompt="Summarize the call in a few sentences.",
            auto_close_message="Thank you for chatting. The conversation has ended.",
            data_storage_setting="everything",
            end_chat_after_silence_ms=3600000,
            language="en-US",
            opt_in_signed_url=True,
            pii_config={
                "categories": ["person_name"],
                "mode": "post_call",
            },
            post_chat_analysis_data=[
                {
                    "description": "The name of the customer.",
                    "name": "customer_name",
                    "type": "string",
                    "examples": ["John Doe", "Jane Smith"],
                }
            ],
            post_chat_analysis_model="gpt-4.1-mini",
            response_engine={
                "llm_id": "llm_234sdertfsdsfsdf",
                "type": "retell-llm",
                "version": 0,
            },
            signed_url_expiration_ms=86400000,
            webhook_timeout_ms=10000,
            webhook_url="https://webhook-url-here",
        )
        assert_matches_type(ChatAgentResponse, chat_agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncRetell) -> None:
        response = await async_client.chat_agent.with_raw_response.update(
            agent_id="16b980523634a6dc504898cda492e939",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        chat_agent = await response.parse()
        assert_matches_type(ChatAgentResponse, chat_agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncRetell) -> None:
        async with async_client.chat_agent.with_streaming_response.update(
            agent_id="16b980523634a6dc504898cda492e939",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            chat_agent = await response.parse()
            assert_matches_type(ChatAgentResponse, chat_agent, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncRetell) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `agent_id` but received ''"):
            await async_client.chat_agent.with_raw_response.update(
                agent_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncRetell) -> None:
        chat_agent = await async_client.chat_agent.list()
        assert_matches_type(ChatAgentListResponse, chat_agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncRetell) -> None:
        chat_agent = await async_client.chat_agent.list(
            limit=50,
            pagination_key="16b980523634a6dc504898cda492e939",
            pagination_key_version=0,
        )
        assert_matches_type(ChatAgentListResponse, chat_agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncRetell) -> None:
        response = await async_client.chat_agent.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        chat_agent = await response.parse()
        assert_matches_type(ChatAgentListResponse, chat_agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncRetell) -> None:
        async with async_client.chat_agent.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            chat_agent = await response.parse()
            assert_matches_type(ChatAgentListResponse, chat_agent, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncRetell) -> None:
        chat_agent = await async_client.chat_agent.delete(
            "oBeDLoLOeuAbiuaMFXRtDOLriTJ5tSxD",
        )
        assert chat_agent is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncRetell) -> None:
        response = await async_client.chat_agent.with_raw_response.delete(
            "oBeDLoLOeuAbiuaMFXRtDOLriTJ5tSxD",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        chat_agent = await response.parse()
        assert chat_agent is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncRetell) -> None:
        async with async_client.chat_agent.with_streaming_response.delete(
            "oBeDLoLOeuAbiuaMFXRtDOLriTJ5tSxD",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            chat_agent = await response.parse()
            assert chat_agent is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncRetell) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `agent_id` but received ''"):
            await async_client.chat_agent.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_get_versions(self, async_client: AsyncRetell) -> None:
        chat_agent = await async_client.chat_agent.get_versions(
            "16b980523634a6dc504898cda492e939",
        )
        assert_matches_type(ChatAgentGetVersionsResponse, chat_agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_get_versions(self, async_client: AsyncRetell) -> None:
        response = await async_client.chat_agent.with_raw_response.get_versions(
            "16b980523634a6dc504898cda492e939",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        chat_agent = await response.parse()
        assert_matches_type(ChatAgentGetVersionsResponse, chat_agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_get_versions(self, async_client: AsyncRetell) -> None:
        async with async_client.chat_agent.with_streaming_response.get_versions(
            "16b980523634a6dc504898cda492e939",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            chat_agent = await response.parse()
            assert_matches_type(ChatAgentGetVersionsResponse, chat_agent, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_get_versions(self, async_client: AsyncRetell) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `agent_id` but received ''"):
            await async_client.chat_agent.with_raw_response.get_versions(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_publish(self, async_client: AsyncRetell) -> None:
        chat_agent = await async_client.chat_agent.publish(
            "16b980523634a6dc504898cda492e939",
        )
        assert chat_agent is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_publish(self, async_client: AsyncRetell) -> None:
        response = await async_client.chat_agent.with_raw_response.publish(
            "16b980523634a6dc504898cda492e939",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        chat_agent = await response.parse()
        assert chat_agent is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_publish(self, async_client: AsyncRetell) -> None:
        async with async_client.chat_agent.with_streaming_response.publish(
            "16b980523634a6dc504898cda492e939",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            chat_agent = await response.parse()
            assert chat_agent is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_publish(self, async_client: AsyncRetell) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `agent_id` but received ''"):
            await async_client.chat_agent.with_raw_response.publish(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, TypedDict

from .._types import SequenceNotStr

__all__ = ["PhoneNumberCreateParams"]


class PhoneNumberCreateParams(TypedDict, total=False):
    area_code: int
    """Area code of the number to obtain.

    Format is a 3 digit integer. Currently only supports US area code.
    """

    country_code: Literal["US", "CA"]
    """The ISO 3166-1 alpha-2 country code of the number you are trying to purchase.

    If left empty, will default to "US".
    """

    inbound_agent_id: Optional[str]
    """Unique id of agent to bind to the number.

    The number will automatically use the agent when receiving inbound calls. If
    null, this number would not accept inbound call.
    """

    inbound_agent_version: Optional[int]
    """Version of the inbound agent to bind to the number.

    If not provided, will default to latest version.
    """

    inbound_allowed_countries: Optional[SequenceNotStr[str]]
    """List of ISO 3166-1 alpha-2 country codes from which inbound calls are allowed.

    If not set or empty, calls from all countries are allowed.
    """

    inbound_webhook_url: Optional[str]
    """
    If set, will send a webhook for inbound calls, where you can to override agent
    id, set dynamic variables and other fields specific to that call.
    """

    nickname: str
    """Nickname of the number. This is for your reference only."""

    number_provider: Literal["twilio", "telnyx"]
    """The provider to purchase the phone number from. Default to twilio."""

    outbound_agent_id: Optional[str]
    """Unique id of agent to bind to the number.

    The number will automatically use the agent when conducting outbound calls. If
    null, this number would not be able to initiate outbound call without agent id
    override.
    """

    outbound_agent_version: Optional[int]
    """Version of the outbound agent to bind to the number.

    If not provided, will default to latest version.
    """

    outbound_allowed_countries: Optional[SequenceNotStr[str]]
    """List of ISO 3166-1 alpha-2 country codes to which outbound calls are allowed.

    If not set or empty, calls to all countries are allowed.
    """

    phone_number: str
    """
    The number you are trying to purchase in E.164 format of the number (+country
    code then number with no space and no special characters).
    """

    toll_free: bool
    """Whether to purchase a toll-free number. Toll-free numbers incur higher costs."""

    transport: Optional[str]
    """Outbound transport protocol to use for the phone number.

    Valid values are "TLS", "TCP" and "UDP". Default is "TCP".
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from typing_extensions import Literal, TypeAlias

from .._models import BaseModel

__all__ = [
    "ChatCreateChatCompletionResponse",
    "Message",
    "MessageMessage",
    "MessageToolCallInvocationMessage",
    "MessageToolCallResultMessage",
    "MessageNodeTransitionMessage",
    "MessageStateTransitionMessage",
]


class MessageMessage(BaseModel):
    content: str
    """Content of the message"""

    created_timestamp: int
    """Create timestamp of the message"""

    message_id: str
    """Unique id of the message"""

    role: Literal["agent", "user"]
    """Documents whether this message is sent by agent or user."""


class MessageToolCallInvocationMessage(BaseModel):
    arguments: str
    """Arguments for this tool call, it's a stringified JSON object."""

    message_id: str
    """Unique id of the message"""

    name: str
    """Name of the function in this tool call."""

    role: Literal["tool_call_invocation"]
    """This is a tool call invocation."""

    tool_call_id: str
    """Tool call id, globally unique."""

    created_timestamp: Optional[int] = None
    """Create timestamp of the message"""


class MessageToolCallResultMessage(BaseModel):
    content: str
    """Result of the tool call, can be a string, a stringified json, etc."""

    created_timestamp: int
    """Create timestamp of the message"""

    message_id: str
    """Unique id of the message"""

    role: Literal["tool_call_result"]
    """This is the result of a tool call."""

    tool_call_id: str
    """Tool call id, globally unique."""


class MessageNodeTransitionMessage(BaseModel):
    created_timestamp: int
    """Create timestamp of the message"""

    message_id: str
    """Unique id of the message"""

    role: Literal["node_transition"]
    """This is a node transition."""

    former_node_id: Optional[str] = None
    """Former node id"""

    former_node_name: Optional[str] = None
    """Former node name"""

    new_node_id: Optional[str] = None
    """New node id"""

    new_node_name: Optional[str] = None
    """New node name"""


class MessageStateTransitionMessage(BaseModel):
    created_timestamp: int
    """Create timestamp of the message"""

    message_id: str
    """Unique id of the message"""

    role: Literal["state_transition"]
    """This is a state transition."""

    former_state_name: Optional[str] = None
    """Former state name"""

    new_state_name: Optional[str] = None
    """New state name"""


Message: TypeAlias = Union[
    MessageMessage,
    MessageToolCallInvocationMessage,
    MessageToolCallResultMessage,
    MessageNodeTransitionMessage,
    MessageStateTransitionMessage,
]


class ChatCreateChatCompletionResponse(BaseModel):
    messages: List[Message]
    """
    New messages generated by the agent during this completion, including any tool
    call invocations and their results. Does not include the original input
    messages.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["BatchCallResponse", "CallTimeWindow", "CallTimeWindowWindow"]


class CallTimeWindowWindow(BaseModel):
    end: float
    """End time in minutes since local midnight."""

    start: float
    """Start time in minutes since local midnight."""


class CallTimeWindow(BaseModel):
    """Canonicalized minutes-based time windows.

    Present only if specified when the batch call was created or updated. See CallTimeWindow for format details ([startMin, endMin) in local minutes; no cross-midnight).
    """

    windows: List[CallTimeWindowWindow]
    """List of TimeWindow (start/end in minutes since local midnight)."""

    day: Optional[List[Literal["Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday"]]] = None
    """Optional list of days to which the windows apply.

    If omitted or empty, windows apply to every day.
    """

    timezone: Optional[str] = None
    """IANA timezone (e.g.

    America/Los_Angeles). Defaults to America/Los_Angeles if omitted.
    """


class BatchCallResponse(BaseModel):
    batch_call_id: str
    """Unique id of the batch call."""

    from_number: str

    name: str

    scheduled_timestamp: float

    total_task_count: float
    """Number of tasks within the batch call"""

    call_time_window: Optional[CallTimeWindow] = None
    """Canonicalized minutes-based time windows.

    Present only if specified when the batch call was created or updated. See
    CallTimeWindow for format details ([startMin, endMin) in local minutes; no
    cross-midnight).
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel

__all__ = ["VoiceSearchResponse", "Voice"]


class Voice(BaseModel):
    """Voices retrieved from the provider."""

    description: Optional[str] = None
    """Description of the voice."""

    name: Optional[str] = None
    """Name of the voice."""

    provider_voice_id: Optional[str] = None
    """id of the voice from the provider."""

    public_user_id: Optional[str] = None
    """For elevenlabs only. User id of the voice owner."""


class VoiceSearchResponse(BaseModel):
    voices: List[Voice]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import test_create_batch_test_params
from .._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.batch_test_response import BatchTestResponse

__all__ = ["TestsResource", "AsyncTestsResource"]


class TestsResource(SyncAPIResource):
    __test__ = False

    @cached_property
    def with_raw_response(self) -> TestsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/RetellAI/retell-python-sdk#accessing-raw-response-data-eg-headers
        """
        return TestsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TestsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/RetellAI/retell-python-sdk#with_streaming_response
        """
        return TestsResourceWithStreamingResponse(self)

    def create_batch_test(
        self,
        *,
        response_engine: test_create_batch_test_params.ResponseEngine,
        test_case_definition_ids: SequenceNotStr[str],
        reserved_concurrency: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BatchTestResponse:
        """
        Create a batch test to run multiple test cases

        Args:
          response_engine: Response engine to use for the test cases. Custom LLM is not supported.

          test_case_definition_ids: Array of test case definition IDs to run

          reserved_concurrency: Number of concurrency reserved for all other calls that are not triggered by
              batch calls, such as inbound calls.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/create-batch-test",
            body=maybe_transform(
                {
                    "response_engine": response_engine,
                    "test_case_definition_ids": test_case_definition_ids,
                    "reserved_concurrency": reserved_concurrency,
                },
                test_create_batch_test_params.TestCreateBatchTestParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BatchTestResponse,
        )


class AsyncTestsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTestsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/RetellAI/retell-python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncTestsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTestsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/RetellAI/retell-python-sdk#with_streaming_response
        """
        return AsyncTestsResourceWithStreamingResponse(self)

    async def create_batch_test(
        self,
        *,
        response_engine: test_create_batch_test_params.ResponseEngine,
        test_case_definition_ids: SequenceNotStr[str],
        reserved_concurrency: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BatchTestResponse:
        """
        Create a batch test to run multiple test cases

        Args:
          response_engine: Response engine to use for the test cases. Custom LLM is not supported.

          test_case_definition_ids: Array of test case definition IDs to run

          reserved_concurrency: Number of concurrency reserved for all other calls that are not triggered by
              batch calls, such as inbound calls.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/create-batch-test",
            body=await async_maybe_transform(
                {
                    "response_engine": response_engine,
                    "test_case_definition_ids": test_case_definition_ids,
                    "reserved_concurrency": reserved_concurrency,
                },
                test_create_batch_test_params.TestCreateBatchTestParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BatchTestResponse,
        )


class TestsResourceWithRawResponse:
    __test__ = False

    def __init__(self, tests: TestsResource) -> None:
        self._tests = tests

        self.create_batch_test = to_raw_response_wrapper(
            tests.create_batch_test,
        )


class AsyncTestsResourceWithRawResponse:
    def __init__(self, tests: AsyncTestsResource) -> None:
        self._tests = tests

        self.create_batch_test = async_to_raw_response_wrapper(
            tests.create_batch_test,
        )


class TestsResourceWithStreamingResponse:
    __test__ = False

    def __init__(self, tests: TestsResource) -> None:
        self._tests = tests

        self.create_batch_test = to_streamed_response_wrapper(
            tests.create_batch_test,
        )


class AsyncTestsResourceWithStreamingResponse:
    def __init__(self, tests: AsyncTestsResource) -> None:
        self._tests = tests

        self.create_batch_test = async_to_streamed_response_wrapper(
            tests.create_batch_test,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["VoiceAddResourceParams"]


class VoiceAddResourceParams(TypedDict, total=False):
    provider_voice_id: Required[str]
    """Voice id assigned by the provider."""

    voice_name: Required[str]
    """A custom name for the voice."""

    public_user_id: str
    """Required for ElevenLabs only. User id of the voice owner."""

    voice_provider: Literal["elevenlabs", "cartesia", "minimax"]
    """Voice provider to add the voice from."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union
from typing_extensions import TypeAlias

from .web_call_response import WebCallResponse
from .phone_call_response import PhoneCallResponse

__all__ = ["CallResponse"]

CallResponse: TypeAlias = Union[WebCallResponse, PhoneCallResponse]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from typing_extensions import Literal, TypeAlias

from .._models import BaseModel

__all__ = [
    "ChatAgentResponse",
    "ResponseEngine",
    "ResponseEngineResponseEngineRetellLm",
    "ResponseEngineResponseEngineCustomLm",
    "ResponseEngineResponseEngineConversationFlow",
    "PiiConfig",
    "PostChatAnalysisData",
    "PostChatAnalysisDataStringAnalysisData",
    "PostChatAnalysisDataEnumAnalysisData",
    "PostChatAnalysisDataBooleanAnalysisData",
    "PostChatAnalysisDataNumberAnalysisData",
]


class ResponseEngineResponseEngineRetellLm(BaseModel):
    llm_id: str
    """id of the Retell LLM Response Engine."""

    type: Literal["retell-llm"]
    """type of the Response Engine."""

    version: Optional[float] = None
    """Version of the Retell LLM Response Engine."""


class ResponseEngineResponseEngineCustomLm(BaseModel):
    llm_websocket_url: str
    """LLM websocket url of the custom LLM."""

    type: Literal["custom-llm"]
    """type of the Response Engine."""


class ResponseEngineResponseEngineConversationFlow(BaseModel):
    conversation_flow_id: str
    """ID of the Conversation Flow Response Engine."""

    type: Literal["conversation-flow"]
    """type of the Response Engine."""

    version: Optional[float] = None
    """Version of the Conversation Flow Response Engine."""


ResponseEngine: TypeAlias = Union[
    ResponseEngineResponseEngineRetellLm,
    ResponseEngineResponseEngineCustomLm,
    ResponseEngineResponseEngineConversationFlow,
]


class PiiConfig(BaseModel):
    """Configuration for PII scrubbing from transcripts and recordings."""

    categories: List[
        Literal[
            "person_name",
            "address",
            "email",
            "phone_number",
            "ssn",
            "passport",
            "driver_license",
            "credit_card",
            "bank_account",
            "password",
            "pin",
            "medical_id",
            "date_of_birth",
            "customer_account_number",
        ]
    ]
    """List of PII categories to scrub from transcripts and recordings."""

    mode: Literal["post_call"]
    """The processing mode for PII scrubbing. Currently only post-call is supported."""


class PostChatAnalysisDataStringAnalysisData(BaseModel):
    description: str
    """Description of the variable."""

    name: str
    """Name of the variable."""

    type: Literal["string"]
    """Type of the variable to extract."""

    examples: Optional[List[str]] = None
    """Examples of the variable value to teach model the style and syntax."""


class PostChatAnalysisDataEnumAnalysisData(BaseModel):
    choices: List[str]
    """The possible values of the variable, must be non empty array."""

    description: str
    """Description of the variable."""

    name: str
    """Name of the variable."""

    type: Literal["enum"]
    """Type of the variable to extract."""


class PostChatAnalysisDataBooleanAnalysisData(BaseModel):
    description: str
    """Description of the variable."""

    name: str
    """Name of the variable."""

    type: Literal["boolean"]
    """Type of the variable to extract."""


class PostChatAnalysisDataNumberAnalysisData(BaseModel):
    description: str
    """Description of the variable."""

    name: str
    """Name of the variable."""

    type: Literal["number"]
    """Type of the variable to extract."""


PostChatAnalysisData: TypeAlias = Union[
    PostChatAnalysisDataStringAnalysisData,
    PostChatAnalysisDataEnumAnalysisData,
    PostChatAnalysisDataBooleanAnalysisData,
    PostChatAnalysisDataNumberAnalysisData,
]


class ChatAgentResponse(BaseModel):
    agent_id: str
    """Unique id of chat agent."""

    last_modification_timestamp: int
    """Last modification timestamp (milliseconds since epoch).

    Either the time of last update or creation if no updates available.
    """

    response_engine: ResponseEngine
    """The Response Engine to attach to the agent.

    It is used to generate responses for the agent. You need to create a Response
    Engine first before attaching it to an agent.
    """

    agent_name: Optional[str] = None
    """The name of the chat agent. Only used for your own reference."""

    analysis_successful_prompt: Optional[str] = None
    """
    The prompt to use for post call analysis to evaluate whether the call is
    successful. Set to null to use the default prompt.
    """

    analysis_summary_prompt: Optional[str] = None
    """The prompt to use for post call analysis to summarize the call.

    Set to null to use the default prompt.
    """

    auto_close_message: Optional[str] = None
    """Message to display when the chat is automatically closed."""

    data_storage_setting: Optional[Literal["everything", "everything_except_pii", "basic_attributes_only"]] = None
    """Controls what data is stored for this agent.

    "everything" stores all data including transcripts and recordings.
    "everything_except_pii" stores data but excludes PII when possible based on PII
    configuration. "basic_attributes_only" stores only basic metadata. If not set,
    defaults to "everything".
    """

    end_chat_after_silence_ms: Optional[int] = None
    """If users stay silent for a period after agent speech, end the chat.

    The minimum value allowed is 360,000 ms (0.1 hours). The maximum value allowed
    is 259,200,000 ms (72 hours). By default, this is set to 3,600,000 (1 hour).
    """

    is_published: Optional[bool] = None
    """Whether the chat agent is published."""

    language: Optional[
        Literal[
            "en-US",
            "en-IN",
            "en-GB",
            "en-AU",
            "en-NZ",
            "de-DE",
            "es-ES",
            "es-419",
            "hi-IN",
            "fr-FR",
            "fr-CA",
            "ja-JP",
            "pt-PT",
            "pt-BR",
            "zh-CN",
            "ru-RU",
            "it-IT",
            "ko-KR",
            "nl-NL",
            "nl-BE",
            "pl-PL",
            "tr-TR",
            "th-TH",
            "vi-VN",
            "ro-RO",
            "bg-BG",
            "ca-ES",
            "da-DK",
            "fi-FI",
            "el-GR",
            "hu-HU",
            "id-ID",
            "no-NO",
            "sk-SK",
            "sv-SE",
            "lt-LT",
            "lv-LV",
            "multi",
        ]
    ] = None
    """Specifies what language (and dialect) the chat will operate in.

    For instance, selecting `en-GB` optimizes for British English. If unset, will
    use default value `en-US`. Select `multi` for multilingual support, currently
    this supports Spanish and English.
    """

    opt_in_signed_url: Optional[bool] = None
    """Whether this agent opts in to signed url for public log.

    If not set, default value of false will apply.
    """

    pii_config: Optional[PiiConfig] = None
    """Configuration for PII scrubbing from transcripts and recordings."""

    post_chat_analysis_data: Optional[List[PostChatAnalysisData]] = None
    """Post chat analysis data to extract from the chat.

    This data will augment the pre-defined variables extracted in the chat analysis.
    This will be available after the chat ends.
    """

    post_chat_analysis_model: Optional[
        Literal[
            "gpt-4.1",
            "gpt-4.1-mini",
            "gpt-4.1-nano",
            "gpt-5",
            "gpt-5.1",
            "gpt-5.2",
            "gpt-5-mini",
            "gpt-5-nano",
            "claude-4.5-sonnet",
            "claude-4.5-haiku",
            "gemini-2.5-flash",
            "gemini-2.5-flash-lite",
            "gemini-3.0-flash",
        ]
    ] = None
    """The model to use for post chat analysis. Default to gpt-4.1-mini."""

    signed_url_expiration_ms: Optional[int] = None
    """The expiration time for the signed url in milliseconds.

    Only applicable when opt_in_signed_url is true. If not set, default value of
    86400000 (24 hours) will apply.
    """

    version: Optional[int] = None
    """The version of the chat agent."""

    webhook_timeout_ms: Optional[int] = None
    """The timeout for the webhook in milliseconds.

    If not set, default value of 10000 will apply.
    """

    webhook_url: Optional[str] = None
    """The webhook for agent to listen to chat events.

    See what events it would get at [webhook doc](/features/webhook). If set, will
    binds webhook events for this agent to the specified url, and will ignore the
    account level webhook for this agent. Set to `null` to remove webhook url from
    this agent.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import TYPE_CHECKING, Any, Mapping
from typing_extensions import Self, override

import httpx

from . import _exceptions
from ._qs import Querystring
from ._types import (
    Omit,
    Timeout,
    NotGiven,
    Transport,
    ProxiesTypes,
    RequestOptions,
    not_given,
)
from ._utils import is_given, get_async_library
from ._compat import cached_property
from ._version import __version__
from ._streaming import Stream as Stream, AsyncStream as AsyncStream
from ._exceptions import APIStatusError
from ._base_client import (
    DEFAULT_MAX_RETRIES,
    SyncAPIClient,
    AsyncAPIClient,
)
from .lib.webhook_auth import verify  # type: ignore

if TYPE_CHECKING:
    from .resources import (
        llm,
        call,
        chat,
        agent,
        tests,
        voice,
        mcp_tool,
        batch_call,
        chat_agent,
        concurrency,
        phone_number,
        knowledge_base,
        conversation_flow,
        conversation_flow_component,
    )
    from .resources.llm import LlmResource, AsyncLlmResource
    from .resources.call import CallResource, AsyncCallResource
    from .resources.chat import ChatResource, AsyncChatResource
    from .resources.agent import AgentResource, AsyncAgentResource
    from .resources.tests import TestsResource, AsyncTestsResource
    from .resources.voice import VoiceResource, AsyncVoiceResource
    from .resources.mcp_tool import McpToolResource, AsyncMcpToolResource
    from .resources.batch_call import BatchCallResource, AsyncBatchCallResource
    from .resources.chat_agent import ChatAgentResource, AsyncChatAgentResource
    from .resources.concurrency import ConcurrencyResource, AsyncConcurrencyResource
    from .resources.phone_number import PhoneNumberResource, AsyncPhoneNumberResource
    from .resources.knowledge_base import KnowledgeBaseResource, AsyncKnowledgeBaseResource
    from .resources.conversation_flow import ConversationFlowResource, AsyncConversationFlowResource
    from .resources.conversation_flow_component import (
        ConversationFlowComponentResource,
        AsyncConversationFlowComponentResource,
    )

__all__ = ["Timeout", "Transport", "ProxiesTypes", "RequestOptions", "Retell", "AsyncRetell", "Client", "AsyncClient"]


class Retell(SyncAPIClient):
    # client options
    api_key: str

    def __init__(
        self,
        *,
        api_key: str,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#client) for more details.
        http_client: httpx.Client | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new synchronous Retell client instance."""
        self.api_key = api_key

        if base_url is None:
            base_url = os.environ.get("RETELL_BASE_URL")
        if base_url is None:
            base_url = f"https://api.retellai.com"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

    @cached_property
    def call(self) -> CallResource:
        from .resources.call import CallResource

        return CallResource(self)

    @cached_property
    def chat(self) -> ChatResource:
        from .resources.chat import ChatResource

        return ChatResource(self)

    @cached_property
    def phone_number(self) -> PhoneNumberResource:
        from .resources.phone_number import PhoneNumberResource

        return PhoneNumberResource(self)

    @cached_property
    def agent(self) -> AgentResource:
        from .resources.agent import AgentResource

        return AgentResource(self)

    @cached_property
    def chat_agent(self) -> ChatAgentResource:
        from .resources.chat_agent import ChatAgentResource

        return ChatAgentResource(self)

    @cached_property
    def llm(self) -> LlmResource:
        from .resources.llm import LlmResource

        return LlmResource(self)

    @cached_property
    def conversation_flow(self) -> ConversationFlowResource:
        from .resources.conversation_flow import ConversationFlowResource

        return ConversationFlowResource(self)

    @cached_property
    def conversation_flow_component(self) -> ConversationFlowComponentResource:
        from .resources.conversation_flow_component import ConversationFlowComponentResource

        return ConversationFlowComponentResource(self)

    @cached_property
    def knowledge_base(self) -> KnowledgeBaseResource:
        from .resources.knowledge_base import KnowledgeBaseResource

        return KnowledgeBaseResource(self)

    @cached_property
    def voice(self) -> VoiceResource:
        from .resources.voice import VoiceResource

        return VoiceResource(self)

    @cached_property
    def concurrency(self) -> ConcurrencyResource:
        from .resources.concurrency import ConcurrencyResource

        return ConcurrencyResource(self)

    @cached_property
    def batch_call(self) -> BatchCallResource:
        from .resources.batch_call import BatchCallResource

        return BatchCallResource(self)

    @cached_property
    def tests(self) -> TestsResource:
        from .resources.tests import TestsResource

        return TestsResource(self)

    @cached_property
    def mcp_tool(self) -> McpToolResource:
        from .resources.mcp_tool import McpToolResource

        return McpToolResource(self)

    @cached_property
    def with_raw_response(self) -> RetellWithRawResponse:
        return RetellWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RetellWithStreamedResponse:
        return RetellWithStreamedResponse(self)

        self.verify = verify  # type: ignore

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="brackets")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_key = self.api_key
        return {"Authorization": f"Bearer {api_key}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": "false",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.Client | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class AsyncRetell(AsyncAPIClient):
    # client options
    api_key: str

    def __init__(
        self,
        *,
        api_key: str,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultAsyncHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#asyncclient) for more details.
        http_client: httpx.AsyncClient | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new async AsyncRetell client instance."""
        self.api_key = api_key

        if base_url is None:
            base_url = os.environ.get("RETELL_BASE_URL")
        if base_url is None:
            base_url = f"https://api.retellai.com"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

    @cached_property
    def call(self) -> AsyncCallResource:
        from .resources.call import AsyncCallResource

        return AsyncCallResource(self)

    @cached_property
    def chat(self) -> AsyncChatResource:
        from .resources.chat import AsyncChatResource

        return AsyncChatResource(self)

    @cached_property
    def phone_number(self) -> AsyncPhoneNumberResource:
        from .resources.phone_number import AsyncPhoneNumberResource

        return AsyncPhoneNumberResource(self)

    @cached_property
    def agent(self) -> AsyncAgentResource:
        from .resources.agent import AsyncAgentResource

        return AsyncAgentResource(self)

    @cached_property
    def chat_agent(self) -> AsyncChatAgentResource:
        from .resources.chat_agent import AsyncChatAgentResource

        return AsyncChatAgentResource(self)

    @cached_property
    def llm(self) -> AsyncLlmResource:
        from .resources.llm import AsyncLlmResource

        return AsyncLlmResource(self)

    @cached_property
    def conversation_flow(self) -> AsyncConversationFlowResource:
        from .resources.conversation_flow import AsyncConversationFlowResource

        return AsyncConversationFlowResource(self)

    @cached_property
    def conversation_flow_component(self) -> AsyncConversationFlowComponentResource:
        from .resources.conversation_flow_component import AsyncConversationFlowComponentResource

        return AsyncConversationFlowComponentResource(self)

    @cached_property
    def knowledge_base(self) -> AsyncKnowledgeBaseResource:
        from .resources.knowledge_base import AsyncKnowledgeBaseResource

        return AsyncKnowledgeBaseResource(self)

    @cached_property
    def voice(self) -> AsyncVoiceResource:
        from .resources.voice import AsyncVoiceResource

        return AsyncVoiceResource(self)

    @cached_property
    def concurrency(self) -> AsyncConcurrencyResource:
        from .resources.concurrency import AsyncConcurrencyResource

        return AsyncConcurrencyResource(self)

    @cached_property
    def batch_call(self) -> AsyncBatchCallResource:
        from .resources.batch_call import AsyncBatchCallResource

        return AsyncBatchCallResource(self)

    @cached_property
    def tests(self) -> AsyncTestsResource:
        from .resources.tests import AsyncTestsResource

        return AsyncTestsResource(self)

    @cached_property
    def mcp_tool(self) -> AsyncMcpToolResource:
        from .resources.mcp_tool import AsyncMcpToolResource

        return AsyncMcpToolResource(self)

    @cached_property
    def with_raw_response(self) -> AsyncRetellWithRawResponse:
        return AsyncRetellWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRetellWithStreamedResponse:
        return AsyncRetellWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="brackets")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_key = self.api_key
        return {"Authorization": f"Bearer {api_key}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": f"async:{get_async_library()}",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.AsyncClient | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class RetellWithRawResponse:
    _client: Retell

    def __init__(self, client: Retell) -> None:
        self._client = client

    @cached_property
    def call(self) -> call.CallResourceWithRawResponse:
        from .resources.call import CallResourceWithRawResponse

        return CallResourceWithRawResponse(self._client.call)

    @cached_property
    def chat(self) -> chat.ChatResourceWithRawResponse:
        from .resources.chat import ChatResourceWithRawResponse

        return ChatResourceWithRawResponse(self._client.chat)

    @cached_property
    def phone_number(self) -> phone_number.PhoneNumberResourceWithRawResponse:
        from .resources.phone_number import PhoneNumberResourceWithRawResponse

        return PhoneNumberResourceWithRawResponse(self._client.phone_number)

    @cached_property
    def agent(self) -> agent.AgentResourceWithRawResponse:
        from .resources.agent import AgentResourceWithRawResponse

        return AgentResourceWithRawResponse(self._client.agent)

    @cached_property
    def chat_agent(self) -> chat_agent.ChatAgentResourceWithRawResponse:
        from .resources.chat_agent import ChatAgentResourceWithRawResponse

        return ChatAgentResourceWithRawResponse(self._client.chat_agent)

    @cached_property
    def llm(self) -> llm.LlmResourceWithRawResponse:
        from .resources.llm import LlmResourceWithRawResponse

        return LlmResourceWithRawResponse(self._client.llm)

    @cached_property
    def conversation_flow(self) -> conversation_flow.ConversationFlowResourceWithRawResponse:
        from .resources.conversation_flow import ConversationFlowResourceWithRawResponse

        return ConversationFlowResourceWithRawResponse(self._client.conversation_flow)

    @cached_property
    def conversation_flow_component(
        self,
    ) -> conversation_flow_component.ConversationFlowComponentResourceWithRawResponse:
        from .resources.conversation_flow_component import ConversationFlowComponentResourceWithRawResponse

        return ConversationFlowComponentResourceWithRawResponse(self._client.conversation_flow_component)

    @cached_property
    def knowledge_base(self) -> knowledge_base.KnowledgeBaseResourceWithRawResponse:
        from .resources.knowledge_base import KnowledgeBaseResourceWithRawResponse

        return KnowledgeBaseResourceWithRawResponse(self._client.knowledge_base)

    @cached_property
    def voice(self) -> voice.VoiceResourceWithRawResponse:
        from .resources.voice import VoiceResourceWithRawResponse

        return VoiceResourceWithRawResponse(self._client.voice)

    @cached_property
    def concurrency(self) -> concurrency.ConcurrencyResourceWithRawResponse:
        from .resources.concurrency import ConcurrencyResourceWithRawResponse

        return ConcurrencyResourceWithRawResponse(self._client.concurrency)

    @cached_property
    def batch_call(self) -> batch_call.BatchCallResourceWithRawResponse:
        from .resources.batch_call import BatchCallResourceWithRawResponse

        return BatchCallResourceWithRawResponse(self._client.batch_call)

    @cached_property
    def tests(self) -> tests.TestsResourceWithRawResponse:
        from .resources.tests import TestsResourceWithRawResponse

        return TestsResourceWithRawResponse(self._client.tests)

    @cached_property
    def mcp_tool(self) -> mcp_tool.McpToolResourceWithRawResponse:
        from .resources.mcp_tool import McpToolResourceWithRawResponse

        return McpToolResourceWithRawResponse(self._client.mcp_tool)


class AsyncRetellWithRawResponse:
    _client: AsyncRetell

    def __init__(self, client: AsyncRetell) -> None:
        self._client = client

    @cached_property
    def call(self) -> call.AsyncCallResourceWithRawResponse:
        from .resources.call import AsyncCallResourceWithRawResponse

        return AsyncCallResourceWithRawResponse(self._client.call)

    @cached_property
    def chat(self) -> chat.AsyncChatResourceWithRawResponse:
        from .resources.chat import AsyncChatResourceWithRawResponse

        return AsyncChatResourceWithRawResponse(self._client.chat)

    @cached_property
    def phone_number(self) -> phone_number.AsyncPhoneNumberResourceWithRawResponse:
        from .resources.phone_number import AsyncPhoneNumberResourceWithRawResponse

        return AsyncPhoneNumberResourceWithRawResponse(self._client.phone_number)

    @cached_property
    def agent(self) -> agent.AsyncAgentResourceWithRawResponse:
        from .resources.agent import AsyncAgentResourceWithRawResponse

        return AsyncAgentResourceWithRawResponse(self._client.agent)

    @cached_property
    def chat_agent(self) -> chat_agent.AsyncChatAgentResourceWithRawResponse:
        from .resources.chat_agent import AsyncChatAgentResourceWithRawResponse

        return AsyncChatAgentResourceWithRawResponse(self._client.chat_agent)

    @cached_property
    def llm(self) -> llm.AsyncLlmResourceWithRawResponse:
        from .resources.llm import AsyncLlmResourceWithRawResponse

        return AsyncLlmResourceWithRawResponse(self._client.llm)

    @cached_property
    def conversation_flow(self) -> conversation_flow.AsyncConversationFlowResourceWithRawResponse:
        from .resources.conversation_flow import AsyncConversationFlowResourceWithRawResponse

        return AsyncConversationFlowResourceWithRawResponse(self._client.conversation_flow)

    @cached_property
    def conversation_flow_component(
        self,
    ) -> conversation_flow_component.AsyncConversationFlowComponentResourceWithRawResponse:
        from .resources.conversation_flow_component import AsyncConversationFlowComponentResourceWithRawResponse

        return AsyncConversationFlowComponentResourceWithRawResponse(self._client.conversation_flow_component)

    @cached_property
    def knowledge_base(self) -> knowledge_base.AsyncKnowledgeBaseResourceWithRawResponse:
        from .resources.knowledge_base import AsyncKnowledgeBaseResourceWithRawResponse

        return AsyncKnowledgeBaseResourceWithRawResponse(self._client.knowledge_base)

    @cached_property
    def voice(self) -> voice.AsyncVoiceResourceWithRawResponse:
        from .resources.voice import AsyncVoiceResourceWithRawResponse

        return AsyncVoiceResourceWithRawResponse(self._client.voice)

    @cached_property
    def concurrency(self) -> concurrency.AsyncConcurrencyResourceWithRawResponse:
        from .resources.concurrency import AsyncConcurrencyResourceWithRawResponse

        return AsyncConcurrencyResourceWithRawResponse(self._client.concurrency)

    @cached_property
    def batch_call(self) -> batch_call.AsyncBatchCallResourceWithRawResponse:
        from .resources.batch_call import AsyncBatchCallResourceWithRawResponse

        return AsyncBatchCallResourceWithRawResponse(self._client.batch_call)

    @cached_property
    def tests(self) -> tests.AsyncTestsResourceWithRawResponse:
        from .resources.tests import AsyncTestsResourceWithRawResponse

        return AsyncTestsResourceWithRawResponse(self._client.tests)

    @cached_property
    def mcp_tool(self) -> mcp_tool.AsyncMcpToolResourceWithRawResponse:
        from .resources.mcp_tool import AsyncMcpToolResourceWithRawResponse

        return AsyncMcpToolResourceWithRawResponse(self._client.mcp_tool)


class RetellWithStreamedResponse:
    _client: Retell

    def __init__(self, client: Retell) -> None:
        self._client = client

    @cached_property
    def call(self) -> call.CallResourceWithStreamingResponse:
        from .resources.call import CallResourceWithStreamingResponse

        return CallResourceWithStreamingResponse(self._client.call)

    @cached_property
    def chat(self) -> chat.ChatResourceWithStreamingResponse:
        from .resources.chat import ChatResourceWithStreamingResponse

        return ChatResourceWithStreamingResponse(self._client.chat)

    @cached_property
    def phone_number(self) -> phone_number.PhoneNumberResourceWithStreamingResponse:
        from .resources.phone_number import PhoneNumberResourceWithStreamingResponse

        return PhoneNumberResourceWithStreamingResponse(self._client.phone_number)

    @cached_property
    def agent(self) -> agent.AgentResourceWithStreamingResponse:
        from .resources.agent import AgentResourceWithStreamingResponse

        return AgentResourceWithStreamingResponse(self._client.agent)

    @cached_property
    def chat_agent(self) -> chat_agent.ChatAgentResourceWithStreamingResponse:
        from .resources.chat_agent import ChatAgentResourceWithStreamingResponse

        return ChatAgentResourceWithStreamingResponse(self._client.chat_agent)

    @cached_property
    def llm(self) -> llm.LlmResourceWithStreamingResponse:
        from .resources.llm import LlmResourceWithStreamingResponse

        return LlmResourceWithStreamingResponse(self._client.llm)

    @cached_property
    def conversation_flow(self) -> conversation_flow.ConversationFlowResourceWithStreamingResponse:
        from .resources.conversation_flow import ConversationFlowResourceWithStreamingResponse

        return ConversationFlowResourceWithStreamingResponse(self._client.conversation_flow)

    @cached_property
    def conversation_flow_component(
        self,
    ) -> conversation_flow_component.ConversationFlowComponentResourceWithStreamingResponse:
        from .resources.conversation_flow_component import ConversationFlowComponentResourceWithStreamingResponse

        return ConversationFlowComponentResourceWithStreamingResponse(self._client.conversation_flow_component)

    @cached_property
    def knowledge_base(self) -> knowledge_base.KnowledgeBaseResourceWithStreamingResponse:
        from .resources.knowledge_base import KnowledgeBaseResourceWithStreamingResponse

        return KnowledgeBaseResourceWithStreamingResponse(self._client.knowledge_base)

    @cached_property
    def voice(self) -> voice.VoiceResourceWithStreamingResponse:
        from .resources.voice import VoiceResourceWithStreamingResponse

        return VoiceResourceWithStreamingResponse(self._client.voice)

    @cached_property
    def concurrency(self) -> concurrency.ConcurrencyResourceWithStreamingResponse:
        from .resources.concurrency import ConcurrencyResourceWithStreamingResponse

        return ConcurrencyResourceWithStreamingResponse(self._client.concurrency)

    @cached_property
    def batch_call(self) -> batch_call.BatchCallResourceWithStreamingResponse:
        from .resources.batch_call import BatchCallResourceWithStreamingResponse

        return BatchCallResourceWithStreamingResponse(self._client.batch_call)

    @cached_property
    def tests(self) -> tests.TestsResourceWithStreamingResponse:
        from .resources.tests import TestsResourceWithStreamingResponse

        return TestsResourceWithStreamingResponse(self._client.tests)

    @cached_property
    def mcp_tool(self) -> mcp_tool.McpToolResourceWithStreamingResponse:
        from .resources.mcp_tool import McpToolResourceWithStreamingResponse

        return McpToolResourceWithStreamingResponse(self._client.mcp_tool)


class AsyncRetellWithStreamedResponse:
    _client: AsyncRetell

    def __init__(self, client: AsyncRetell) -> None:
        self._client = client

    @cached_property
    def call(self) -> call.AsyncCallResourceWithStreamingResponse:
        from .resources.call import AsyncCallResourceWithStreamingResponse

        return AsyncCallResourceWithStreamingResponse(self._client.call)

    @cached_property
    def chat(self) -> chat.AsyncChatResourceWithStreamingResponse:
        from .resources.chat import AsyncChatResourceWithStreamingResponse

        return AsyncChatResourceWithStreamingResponse(self._client.chat)

    @cached_property
    def phone_number(self) -> phone_number.AsyncPhoneNumberResourceWithStreamingResponse:
        from .resources.phone_number import AsyncPhoneNumberResourceWithStreamingResponse

        return AsyncPhoneNumberResourceWithStreamingResponse(self._client.phone_number)

    @cached_property
    def agent(self) -> agent.AsyncAgentResourceWithStreamingResponse:
        from .resources.agent import AsyncAgentResourceWithStreamingResponse

        return AsyncAgentResourceWithStreamingResponse(self._client.agent)

    @cached_property
    def chat_agent(self) -> chat_agent.AsyncChatAgentResourceWithStreamingResponse:
        from .resources.chat_agent import AsyncChatAgentResourceWithStreamingResponse

        return AsyncChatAgentResourceWithStreamingResponse(self._client.chat_agent)

    @cached_property
    def llm(self) -> llm.AsyncLlmResourceWithStreamingResponse:
        from .resources.llm import AsyncLlmResourceWithStreamingResponse

        return AsyncLlmResourceWithStreamingResponse(self._client.llm)

    @cached_property
    def conversation_flow(self) -> conversation_flow.AsyncConversationFlowResourceWithStreamingResponse:
        from .resources.conversation_flow import AsyncConversationFlowResourceWithStreamingResponse

        return AsyncConversationFlowResourceWithStreamingResponse(self._client.conversation_flow)

    @cached_property
    def conversation_flow_component(
        self,
    ) -> conversation_flow_component.AsyncConversationFlowComponentResourceWithStreamingResponse:
        from .resources.conversation_flow_component import AsyncConversationFlowComponentResourceWithStreamingResponse

        return AsyncConversationFlowComponentResourceWithStreamingResponse(self._client.conversation_flow_component)

    @cached_property
    def knowledge_base(self) -> knowledge_base.AsyncKnowledgeBaseResourceWithStreamingResponse:
        from .resources.knowledge_base import AsyncKnowledgeBaseResourceWithStreamingResponse

        return AsyncKnowledgeBaseResourceWithStreamingResponse(self._client.knowledge_base)

    @cached_property
    def voice(self) -> voice.AsyncVoiceResourceWithStreamingResponse:
        from .resources.voice import AsyncVoiceResourceWithStreamingResponse

        return AsyncVoiceResourceWithStreamingResponse(self._client.voice)

    @cached_property
    def concurrency(self) -> concurrency.AsyncConcurrencyResourceWithStreamingResponse:
        from .resources.concurrency import AsyncConcurrencyResourceWithStreamingResponse

        return AsyncConcurrencyResourceWithStreamingResponse(self._client.concurrency)

    @cached_property
    def batch_call(self) -> batch_call.AsyncBatchCallResourceWithStreamingResponse:
        from .resources.batch_call import AsyncBatchCallResourceWithStreamingResponse

        return AsyncBatchCallResourceWithStreamingResponse(self._client.batch_call)

    @cached_property
    def tests(self) -> tests.AsyncTestsResourceWithStreamingResponse:
        from .resources.tests import AsyncTestsResourceWithStreamingResponse

        return AsyncTestsResourceWithStreamingResponse(self._client.tests)

    @cached_property
    def mcp_tool(self) -> mcp_tool.AsyncMcpToolResourceWithStreamingResponse:
        from .resources.mcp_tool import AsyncMcpToolResourceWithStreamingResponse

        return AsyncMcpToolResourceWithStreamingResponse(self._client.mcp_tool)


Client = Retell

AsyncClient = AsyncRetell



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import TypedDict

from .._types import SequenceNotStr

__all__ = ["PhoneNumberUpdateParams"]


class PhoneNumberUpdateParams(TypedDict, total=False):
    auth_password: str
    """
    The password used for authentication for the SIP trunk to update for the phone
    number.
    """

    auth_username: str
    """
    The username used for authentication for the SIP trunk to update for the phone
    number.
    """

    inbound_agent_id: Optional[str]
    """Unique id of agent to bind to the number.

    The number will automatically use the agent when receiving inbound calls. If set
    to null, this number would not accept inbound call.
    """

    inbound_agent_version: Optional[int]
    """Version of the inbound agent to bind to the number.

    If not provided, will default to latest version.
    """

    inbound_allowed_countries: Optional[SequenceNotStr[str]]
    """List of ISO 3166-1 alpha-2 country codes from which inbound calls are allowed.

    If not set or empty, calls from all countries are allowed.
    """

    inbound_webhook_url: Optional[str]
    """
    If set, will send a webhook for inbound calls, where you can to override agent
    id, set dynamic variables and other fields specific to that call.
    """

    nickname: Optional[str]
    """Nickname of the number. This is for your reference only."""

    outbound_agent_id: Optional[str]
    """Unique id of agent to bind to the number.

    The number will automatically use the agent when conducting outbound calls. If
    set to null, this number would not be able to initiate outbound call without
    agent id override.
    """

    outbound_agent_version: Optional[int]
    """Version of the outbound agent to bind to the number.

    If not provided, will default to latest version.
    """

    outbound_allowed_countries: Optional[SequenceNotStr[str]]
    """List of ISO 3166-1 alpha-2 country codes to which outbound calls are allowed.

    If not set or empty, calls to all countries are allowed.
    """

    termination_uri: str
    """The termination uri to update for the phone number.

    This is used for outbound calls.
    """

    transport: Optional[str]
    """Outbound transport protocol to update for the phone number.

    Valid values are "TLS", "TCP" and "UDP". Default is "TCP".
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["AgentListParams"]


class AgentListParams(TypedDict, total=False):
    limit: int
    """A limit on the number of objects to be returned.

    Limit can range between 1 and 1000, and the default is 1000.
    """

    pagination_key: str
    """The pagination key to continue fetching the next page of agents.

    Pagination key is represented by a agent id, pagination key and version pair is
    exclusive (not included in the fetched page). If not set, will start from the
    beginning.
    """

    pagination_key_version: int
    """Specifies the version of the agent associated with the pagination_key.

    When paginating, both the pagination_key and its version must be provided to
    ensure consistent ordering and to fetch the next page correctly.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["ConversationFlowListParams"]


class ConversationFlowListParams(TypedDict, total=False):
    limit: int
    """Limit the number of conversation flows returned.

    Default 1000, Max 1000. To retrieve more than 1000, use pagination_key to
    continue fetching the next page.
    """

    pagination_key: str
    """The pagination key to continue fetching the next page of conversation flows.

    Pagination key is represented by a conversation flow id here, and it's exclusive
    (not included in the fetched conversation flows). The last conversation flow id
    from the list conversation flows is usually used as pagination key here. If not
    set, will start from the beginning.
    """

    pagination_key_version: int
    """
    Specifies the version of the conversation flow associated with the
    pagination_key. When paginating, both the pagination_key and its version must be
    provided to ensure consistent ordering and to fetch the next page correctly.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable, Optional
from typing_extensions import Literal

import httpx

from ..types import (
    chat_agent_list_params,
    chat_agent_create_params,
    chat_agent_update_params,
    chat_agent_retrieve_params,
)
from .._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.chat_agent_response import ChatAgentResponse
from ..types.chat_agent_list_response import ChatAgentListResponse
from ..types.chat_agent_get_versions_response import ChatAgentGetVersionsResponse

__all__ = ["ChatAgentResource", "AsyncChatAgentResource"]


class ChatAgentResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ChatAgentResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/RetellAI/retell-python-sdk#accessing-raw-response-data-eg-headers
        """
        return ChatAgentResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ChatAgentResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/RetellAI/retell-python-sdk#with_streaming_response
        """
        return ChatAgentResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        response_engine: chat_agent_create_params.ResponseEngine,
        agent_name: Optional[str] | Omit = omit,
        analysis_successful_prompt: Optional[str] | Omit = omit,
        analysis_summary_prompt: Optional[str] | Omit = omit,
        auto_close_message: Optional[str] | Omit = omit,
        data_storage_setting: Optional[Literal["everything", "everything_except_pii", "basic_attributes_only"]]
        | Omit = omit,
        end_chat_after_silence_ms: int | Omit = omit,
        language: Literal[
            "en-US",
            "en-IN",
            "en-GB",
            "en-AU",
            "en-NZ",
            "de-DE",
            "es-ES",
            "es-419",
            "hi-IN",
            "fr-FR",
            "fr-CA",
            "ja-JP",
            "pt-PT",
            "pt-BR",
            "zh-CN",
            "ru-RU",
            "it-IT",
            "ko-KR",
            "nl-NL",
            "nl-BE",
            "pl-PL",
            "tr-TR",
            "th-TH",
            "vi-VN",
            "ro-RO",
            "bg-BG",
            "ca-ES",
            "da-DK",
            "fi-FI",
            "el-GR",
            "hu-HU",
            "id-ID",
            "no-NO",
            "sk-SK",
            "sv-SE",
            "lt-LT",
            "lv-LV",
            "multi",
        ]
        | Omit = omit,
        opt_in_signed_url: bool | Omit = omit,
        pii_config: chat_agent_create_params.PiiConfig | Omit = omit,
        post_chat_analysis_data: Optional[Iterable[chat_agent_create_params.PostChatAnalysisData]] | Omit = omit,
        post_chat_analysis_model: Optional[
            Literal[
                "gpt-4.1",
                "gpt-4.1-mini",
                "gpt-4.1-nano",
                "gpt-5",
                "gpt-5.1",
                "gpt-5.2",
                "gpt-5-mini",
                "gpt-5-nano",
                "claude-4.5-sonnet",
                "claude-4.5-haiku",
                "gemini-2.5-flash",
                "gemini-2.5-flash-lite",
                "gemini-3.0-flash",
            ]
        ]
        | Omit = omit,
        signed_url_expiration_ms: Optional[int] | Omit = omit,
        webhook_timeout_ms: int | Omit = omit,
        webhook_url: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ChatAgentResponse:
        """
        Create a new chat agent

        Args:
          response_engine: The Response Engine to attach to the agent. It is used to generate responses for
              the agent. You need to create a Response Engine first before attaching it to an
              agent.

          agent_name: The name of the chat agent. Only used for your own reference.

          analysis_successful_prompt: The prompt to use for post call analysis to evaluate whether the call is
              successful. Set to null to use the default prompt.

          analysis_summary_prompt: The prompt to use for post call analysis to summarize the call. Set to null to
              use the default prompt.

          auto_close_message: Message to display when the chat is automatically closed.

          data_storage_setting: Controls what data is stored for this agent. "everything" stores all data
              including transcripts and recordings. "everything_except_pii" stores data but
              excludes PII when possible based on PII configuration. "basic_attributes_only"
              stores only basic metadata. If not set, defaults to "everything".

          end_chat_after_silence_ms: If users stay silent for a period after agent speech, end the chat. The minimum
              value allowed is 360,000 ms (0.1 hours). The maximum value allowed is
              259,200,000 ms (72 hours). By default, this is set to 3,600,000 (1 hour).

          language: Specifies what language (and dialect) the chat will operate in. For instance,
              selecting `en-GB` optimizes for British English. If unset, will use default
              value `en-US`. Select `multi` for multilingual support, currently this supports
              Spanish and English.

          opt_in_signed_url: Whether this agent opts in to signed url for public log. If not set, default
              value of false will apply.

          pii_config: Configuration for PII scrubbing from transcripts and recordings.

          post_chat_analysis_data: Post chat analysis data to extract from the chat. This data will augment the
              pre-defined variables extracted in the chat analysis. This will be available
              after the chat ends.

          post_chat_analysis_model: The model to use for post chat analysis. Default to gpt-4.1-mini.

          signed_url_expiration_ms: The expiration time for the signed url in milliseconds. Only applicable when
              opt_in_signed_url is true. If not set, default value of 86400000 (24 hours) will
              apply.

          webhook_timeout_ms: The timeout for the webhook in milliseconds. If not set, default value of 10000
              will apply.

          webhook_url: The webhook for agent to listen to chat events. See what events it would get at
              [webhook doc](/features/webhook). If set, will binds webhook events for this
              agent to the specified url, and will ignore the account level webhook for this
              agent. Set to `null` to remove webhook url from this agent.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/create-chat-agent",
            body=maybe_transform(
                {
                    "response_engine": response_engine,
                    "agent_name": agent_name,
                    "analysis_successful_prompt": analysis_successful_prompt,
                    "analysis_summary_prompt": analysis_summary_prompt,
                    "auto_close_message": auto_close_message,
                    "data_storage_setting": data_storage_setting,
                    "end_chat_after_silence_ms": end_chat_after_silence_ms,
                    "language": language,
                    "opt_in_signed_url": opt_in_signed_url,
                    "pii_config": pii_config,
                    "post_chat_analysis_data": post_chat_analysis_data,
                    "post_chat_analysis_model": post_chat_analysis_model,
                    "signed_url_expiration_ms": signed_url_expiration_ms,
                    "webhook_timeout_ms": webhook_timeout_ms,
                    "webhook_url": webhook_url,
                },
                chat_agent_create_params.ChatAgentCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ChatAgentResponse,
        )

    def retrieve(
        self,
        agent_id: str,
        *,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ChatAgentResponse:
        """
        Retrieve details of a specific chat agent

        Args:
          version: Optional version of the API to use for this request. If not provided, will
              default to latest version.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        return self._get(
            f"/get-chat-agent/{agent_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"version": version}, chat_agent_retrieve_params.ChatAgentRetrieveParams),
            ),
            cast_to=ChatAgentResponse,
        )

    def update(
        self,
        agent_id: str,
        *,
        version: int | Omit = omit,
        agent_name: Optional[str] | Omit = omit,
        analysis_successful_prompt: Optional[str] | Omit = omit,
        analysis_summary_prompt: Optional[str] | Omit = omit,
        auto_close_message: Optional[str] | Omit = omit,
        data_storage_setting: Optional[Literal["everything", "everything_except_pii", "basic_attributes_only"]]
        | Omit = omit,
        end_chat_after_silence_ms: int | Omit = omit,
        language: Literal[
            "en-US",
            "en-IN",
            "en-GB",
            "en-AU",
            "en-NZ",
            "de-DE",
            "es-ES",
            "es-419",
            "hi-IN",
            "fr-FR",
            "fr-CA",
            "ja-JP",
            "pt-PT",
            "pt-BR",
            "zh-CN",
            "ru-RU",
            "it-IT",
            "ko-KR",
            "nl-NL",
            "nl-BE",
            "pl-PL",
            "tr-TR",
            "th-TH",
            "vi-VN",
            "ro-RO",
            "bg-BG",
            "ca-ES",
            "da-DK",
            "fi-FI",
            "el-GR",
            "hu-HU",
            "id-ID",
            "no-NO",
            "sk-SK",
            "sv-SE",
            "lt-LT",
            "lv-LV",
            "multi",
        ]
        | Omit = omit,
        opt_in_signed_url: bool | Omit = omit,
        pii_config: chat_agent_update_params.PiiConfig | Omit = omit,
        post_chat_analysis_data: Optional[Iterable[chat_agent_update_params.PostChatAnalysisData]] | Omit = omit,
        post_chat_analysis_model: Optional[
            Literal[
                "gpt-4.1",
                "gpt-4.1-mini",
                "gpt-4.1-nano",
                "gpt-5",
                "gpt-5.1",
                "gpt-5.2",
                "gpt-5-mini",
                "gpt-5-nano",
                "claude-4.5-sonnet",
                "claude-4.5-haiku",
                "gemini-2.5-flash",
                "gemini-2.5-flash-lite",
                "gemini-3.0-flash",
            ]
        ]
        | Omit = omit,
        response_engine: chat_agent_update_params.ResponseEngine | Omit = omit,
        signed_url_expiration_ms: Optional[int] | Omit = omit,
        webhook_timeout_ms: int | Omit = omit,
        webhook_url: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ChatAgentResponse:
        """
        Update an existing chat agent

        Args:
          version: Optional version of the API to use for this request. Default to latest version.

          agent_name: The name of the chat agent. Only used for your own reference.

          analysis_successful_prompt: The prompt to use for post call analysis to evaluate whether the call is
              successful. Set to null to use the default prompt.

          analysis_summary_prompt: The prompt to use for post call analysis to summarize the call. Set to null to
              use the default prompt.

          auto_close_message: Message to display when the chat is automatically closed.

          data_storage_setting: Controls what data is stored for this agent. "everything" stores all data
              including transcripts and recordings. "everything_except_pii" stores data but
              excludes PII when possible based on PII configuration. "basic_attributes_only"
              stores only basic metadata. If not set, defaults to "everything".

          end_chat_after_silence_ms: If users stay silent for a period after agent speech, end the chat. The minimum
              value allowed is 360,000 ms (0.1 hours). The maximum value allowed is
              259,200,000 ms (72 hours). By default, this is set to 3,600,000 (1 hour).

          language: Specifies what language (and dialect) the chat will operate in. For instance,
              selecting `en-GB` optimizes for British English. If unset, will use default
              value `en-US`. Select `multi` for multilingual support, currently this supports
              Spanish and English.

          opt_in_signed_url: Whether this agent opts in to signed url for public log. If not set, default
              value of false will apply.

          pii_config: Configuration for PII scrubbing from transcripts and recordings.

          post_chat_analysis_data: Post chat analysis data to extract from the chat. This data will augment the
              pre-defined variables extracted in the chat analysis. This will be available
              after the chat ends.

          post_chat_analysis_model: The model to use for post chat analysis. Default to gpt-4.1-mini.

          response_engine: The Response Engine to attach to the agent. It is used to generate responses for
              the agent. You need to create a Response Engine first before attaching it to an
              agent.

          signed_url_expiration_ms: The expiration time for the signed url in milliseconds. Only applicable when
              opt_in_signed_url is true. If not set, default value of 86400000 (24 hours) will
              apply.

          webhook_timeout_ms: The timeout for the webhook in milliseconds. If not set, default value of 10000
              will apply.

          webhook_url: The webhook for agent to listen to chat events. See what events it would get at
              [webhook doc](/features/webhook). If set, will binds webhook events for this
              agent to the specified url, and will ignore the account level webhook for this
              agent. Set to `null` to remove webhook url from this agent.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        return self._patch(
            f"/update-chat-agent/{agent_id}",
            body=maybe_transform(
                {
                    "agent_name": agent_name,
                    "analysis_successful_prompt": analysis_successful_prompt,
                    "analysis_summary_prompt": analysis_summary_prompt,
                    "auto_close_message": auto_close_message,
                    "data_storage_setting": data_storage_setting,
                    "end_chat_after_silence_ms": end_chat_after_silence_ms,
                    "language": language,
                    "opt_in_signed_url": opt_in_signed_url,
                    "pii_config": pii_config,
                    "post_chat_analysis_data": post_chat_analysis_data,
                    "post_chat_analysis_model": post_chat_analysis_model,
                    "response_engine": response_engine,
                    "signed_url_expiration_ms": signed_url_expiration_ms,
                    "webhook_timeout_ms": webhook_timeout_ms,
                    "webhook_url": webhook_url,
                },
                chat_agent_update_params.ChatAgentUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"version": version}, chat_agent_update_params.ChatAgentUpdateParams),
            ),
            cast_to=ChatAgentResponse,
        )

    def list(
        self,
        *,
        limit: int | Omit = omit,
        pagination_key: str | Omit = omit,
        pagination_key_version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ChatAgentListResponse:
        """
        List all chat agents

        Args:
          limit: A limit on the number of objects to be returned. Limit can range between 1 and
              1000, and the default is 1000.

          pagination_key: The pagination key to continue fetching the next page of agents. Pagination key
              is represented by a agent id, pagination key and version pair is exclusive (not
              included in the fetched page). If not set, will start from the beginning.

          pagination_key_version: Specifies the version of the agent associated with the pagination_key. When
              paginating, both the pagination_key and its version must be provided to ensure
              consistent ordering and to fetch the next page correctly.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/list-chat-agents",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "pagination_key": pagination_key,
                        "pagination_key_version": pagination_key_version,
                    },
                    chat_agent_list_params.ChatAgentListParams,
                ),
            ),
            cast_to=ChatAgentListResponse,
        )

    def delete(
        self,
        agent_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete an existing chat agent

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/delete-chat-agent/{agent_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def get_versions(
        self,
        agent_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ChatAgentGetVersionsResponse:
        """
        Get all versions of a chat agent

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        return self._get(
            f"/get-chat-agent-versions/{agent_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ChatAgentGetVersionsResponse,
        )

    def publish(
        self,
        agent_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Publish the latest version of the chat agent and create a new draft chat agent
        with newer version.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._post(
            f"/publish-chat-agent/{agent_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncChatAgentResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncChatAgentResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/RetellAI/retell-python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncChatAgentResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncChatAgentResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/RetellAI/retell-python-sdk#with_streaming_response
        """
        return AsyncChatAgentResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        response_engine: chat_agent_create_params.ResponseEngine,
        agent_name: Optional[str] | Omit = omit,
        analysis_successful_prompt: Optional[str] | Omit = omit,
        analysis_summary_prompt: Optional[str] | Omit = omit,
        auto_close_message: Optional[str] | Omit = omit,
        data_storage_setting: Optional[Literal["everything", "everything_except_pii", "basic_attributes_only"]]
        | Omit = omit,
        end_chat_after_silence_ms: int | Omit = omit,
        language: Literal[
            "en-US",
            "en-IN",
            "en-GB",
            "en-AU",
            "en-NZ",
            "de-DE",
            "es-ES",
            "es-419",
            "hi-IN",
            "fr-FR",
            "fr-CA",
            "ja-JP",
            "pt-PT",
            "pt-BR",
            "zh-CN",
            "ru-RU",
            "it-IT",
            "ko-KR",
            "nl-NL",
            "nl-BE",
            "pl-PL",
            "tr-TR",
            "th-TH",
            "vi-VN",
            "ro-RO",
            "bg-BG",
            "ca-ES",
            "da-DK",
            "fi-FI",
            "el-GR",
            "hu-HU",
            "id-ID",
            "no-NO",
            "sk-SK",
            "sv-SE",
            "lt-LT",
            "lv-LV",
            "multi",
        ]
        | Omit = omit,
        opt_in_signed_url: bool | Omit = omit,
        pii_config: chat_agent_create_params.PiiConfig | Omit = omit,
        post_chat_analysis_data: Optional[Iterable[chat_agent_create_params.PostChatAnalysisData]] | Omit = omit,
        post_chat_analysis_model: Optional[
            Literal[
                "gpt-4.1",
                "gpt-4.1-mini",
                "gpt-4.1-nano",
                "gpt-5",
                "gpt-5.1",
                "gpt-5.2",
                "gpt-5-mini",
                "gpt-5-nano",
                "claude-4.5-sonnet",
                "claude-4.5-haiku",
                "gemini-2.5-flash",
                "gemini-2.5-flash-lite",
                "gemini-3.0-flash",
            ]
        ]
        | Omit = omit,
        signed_url_expiration_ms: Optional[int] | Omit = omit,
        webhook_timeout_ms: int | Omit = omit,
        webhook_url: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ChatAgentResponse:
        """
        Create a new chat agent

        Args:
          response_engine: The Response Engine to attach to the agent. It is used to generate responses for
              the agent. You need to create a Response Engine first before attaching it to an
              agent.

          agent_name: The name of the chat agent. Only used for your own reference.

          analysis_successful_prompt: The prompt to use for post call analysis to evaluate whether the call is
              successful. Set to null to use the default prompt.

          analysis_summary_prompt: The prompt to use for post call analysis to summarize the call. Set to null to
              use the default prompt.

          auto_close_message: Message to display when the chat is automatically closed.

          data_storage_setting: Controls what data is stored for this agent. "everything" stores all data
              including transcripts and recordings. "everything_except_pii" stores data but
              excludes PII when possible based on PII configuration. "basic_attributes_only"
              stores only basic metadata. If not set, defaults to "everything".

          end_chat_after_silence_ms: If users stay silent for a period after agent speech, end the chat. The minimum
              value allowed is 360,000 ms (0.1 hours). The maximum value allowed is
              259,200,000 ms (72 hours). By default, this is set to 3,600,000 (1 hour).

          language: Specifies what language (and dialect) the chat will operate in. For instance,
              selecting `en-GB` optimizes for British English. If unset, will use default
              value `en-US`. Select `multi` for multilingual support, currently this supports
              Spanish and English.

          opt_in_signed_url: Whether this agent opts in to signed url for public log. If not set, default
              value of false will apply.

          pii_config: Configuration for PII scrubbing from transcripts and recordings.

          post_chat_analysis_data: Post chat analysis data to extract from the chat. This data will augment the
              pre-defined variables extracted in the chat analysis. This will be available
              after the chat ends.

          post_chat_analysis_model: The model to use for post chat analysis. Default to gpt-4.1-mini.

          signed_url_expiration_ms: The expiration time for the signed url in milliseconds. Only applicable when
              opt_in_signed_url is true. If not set, default value of 86400000 (24 hours) will
              apply.

          webhook_timeout_ms: The timeout for the webhook in milliseconds. If not set, default value of 10000
              will apply.

          webhook_url: The webhook for agent to listen to chat events. See what events it would get at
              [webhook doc](/features/webhook). If set, will binds webhook events for this
              agent to the specified url, and will ignore the account level webhook for this
              agent. Set to `null` to remove webhook url from this agent.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/create-chat-agent",
            body=await async_maybe_transform(
                {
                    "response_engine": response_engine,
                    "agent_name": agent_name,
                    "analysis_successful_prompt": analysis_successful_prompt,
                    "analysis_summary_prompt": analysis_summary_prompt,
                    "auto_close_message": auto_close_message,
                    "data_storage_setting": data_storage_setting,
                    "end_chat_after_silence_ms": end_chat_after_silence_ms,
                    "language": language,
                    "opt_in_signed_url": opt_in_signed_url,
                    "pii_config": pii_config,
                    "post_chat_analysis_data": post_chat_analysis_data,
                    "post_chat_analysis_model": post_chat_analysis_model,
                    "signed_url_expiration_ms": signed_url_expiration_ms,
                    "webhook_timeout_ms": webhook_timeout_ms,
                    "webhook_url": webhook_url,
                },
                chat_agent_create_params.ChatAgentCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ChatAgentResponse,
        )

    async def retrieve(
        self,
        agent_id: str,
        *,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ChatAgentResponse:
        """
        Retrieve details of a specific chat agent

        Args:
          version: Optional version of the API to use for this request. If not provided, will
              default to latest version.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        return await self._get(
            f"/get-chat-agent/{agent_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"version": version}, chat_agent_retrieve_params.ChatAgentRetrieveParams
                ),
            ),
            cast_to=ChatAgentResponse,
        )

    async def update(
        self,
        agent_id: str,
        *,
        version: int | Omit = omit,
        agent_name: Optional[str] | Omit = omit,
        analysis_successful_prompt: Optional[str] | Omit = omit,
        analysis_summary_prompt: Optional[str] | Omit = omit,
        auto_close_message: Optional[str] | Omit = omit,
        data_storage_setting: Optional[Literal["everything", "everything_except_pii", "basic_attributes_only"]]
        | Omit = omit,
        end_chat_after_silence_ms: int | Omit = omit,
        language: Literal[
            "en-US",
            "en-IN",
            "en-GB",
            "en-AU",
            "en-NZ",
            "de-DE",
            "es-ES",
            "es-419",
            "hi-IN",
            "fr-FR",
            "fr-CA",
            "ja-JP",
            "pt-PT",
            "pt-BR",
            "zh-CN",
            "ru-RU",
            "it-IT",
            "ko-KR",
            "nl-NL",
            "nl-BE",
            "pl-PL",
            "tr-TR",
            "th-TH",
            "vi-VN",
            "ro-RO",
            "bg-BG",
            "ca-ES",
            "da-DK",
            "fi-FI",
            "el-GR",
            "hu-HU",
            "id-ID",
            "no-NO",
            "sk-SK",
            "sv-SE",
            "lt-LT",
            "lv-LV",
            "multi",
        ]
        | Omit = omit,
        opt_in_signed_url: bool | Omit = omit,
        pii_config: chat_agent_update_params.PiiConfig | Omit = omit,
        post_chat_analysis_data: Optional[Iterable[chat_agent_update_params.PostChatAnalysisData]] | Omit = omit,
        post_chat_analysis_model: Optional[
            Literal[
                "gpt-4.1",
                "gpt-4.1-mini",
                "gpt-4.1-nano",
                "gpt-5",
                "gpt-5.1",
                "gpt-5.2",
                "gpt-5-mini",
                "gpt-5-nano",
                "claude-4.5-sonnet",
                "claude-4.5-haiku",
                "gemini-2.5-flash",
                "gemini-2.5-flash-lite",
                "gemini-3.0-flash",
            ]
        ]
        | Omit = omit,
        response_engine: chat_agent_update_params.ResponseEngine | Omit = omit,
        signed_url_expiration_ms: Optional[int] | Omit = omit,
        webhook_timeout_ms: int | Omit = omit,
        webhook_url: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ChatAgentResponse:
        """
        Update an existing chat agent

        Args:
          version: Optional version of the API to use for this request. Default to latest version.

          agent_name: The name of the chat agent. Only used for your own reference.

          analysis_successful_prompt: The prompt to use for post call analysis to evaluate whether the call is
              successful. Set to null to use the default prompt.

          analysis_summary_prompt: The prompt to use for post call analysis to summarize the call. Set to null to
              use the default prompt.

          auto_close_message: Message to display when the chat is automatically closed.

          data_storage_setting: Controls what data is stored for this agent. "everything" stores all data
              including transcripts and recordings. "everything_except_pii" stores data but
              excludes PII when possible based on PII configuration. "basic_attributes_only"
              stores only basic metadata. If not set, defaults to "everything".

          end_chat_after_silence_ms: If users stay silent for a period after agent speech, end the chat. The minimum
              value allowed is 360,000 ms (0.1 hours). The maximum value allowed is
              259,200,000 ms (72 hours). By default, this is set to 3,600,000 (1 hour).

          language: Specifies what language (and dialect) the chat will operate in. For instance,
              selecting `en-GB` optimizes for British English. If unset, will use default
              value `en-US`. Select `multi` for multilingual support, currently this supports
              Spanish and English.

          opt_in_signed_url: Whether this agent opts in to signed url for public log. If not set, default
              value of false will apply.

          pii_config: Configuration for PII scrubbing from transcripts and recordings.

          post_chat_analysis_data: Post chat analysis data to extract from the chat. This data will augment the
              pre-defined variables extracted in the chat analysis. This will be available
              after the chat ends.

          post_chat_analysis_model: The model to use for post chat analysis. Default to gpt-4.1-mini.

          response_engine: The Response Engine to attach to the agent. It is used to generate responses for
              the agent. You need to create a Response Engine first before attaching it to an
              agent.

          signed_url_expiration_ms: The expiration time for the signed url in milliseconds. Only applicable when
              opt_in_signed_url is true. If not set, default value of 86400000 (24 hours) will
              apply.

          webhook_timeout_ms: The timeout for the webhook in milliseconds. If not set, default value of 10000
              will apply.

          webhook_url: The webhook for agent to listen to chat events. See what events it would get at
              [webhook doc](/features/webhook). If set, will binds webhook events for this
              agent to the specified url, and will ignore the account level webhook for this
              agent. Set to `null` to remove webhook url from this agent.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        return await self._patch(
            f"/update-chat-agent/{agent_id}",
            body=await async_maybe_transform(
                {
                    "agent_name": agent_name,
                    "analysis_successful_prompt": analysis_successful_prompt,
                    "analysis_summary_prompt": analysis_summary_prompt,
                    "auto_close_message": auto_close_message,
                    "data_storage_setting": data_storage_setting,
                    "end_chat_after_silence_ms": end_chat_after_silence_ms,
                    "language": language,
                    "opt_in_signed_url": opt_in_signed_url,
                    "pii_config": pii_config,
                    "post_chat_analysis_data": post_chat_analysis_data,
                    "post_chat_analysis_model": post_chat_analysis_model,
                    "response_engine": response_engine,
                    "signed_url_expiration_ms": signed_url_expiration_ms,
                    "webhook_timeout_ms": webhook_timeout_ms,
                    "webhook_url": webhook_url,
                },
                chat_agent_update_params.ChatAgentUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"version": version}, chat_agent_update_params.ChatAgentUpdateParams),
            ),
            cast_to=ChatAgentResponse,
        )

    async def list(
        self,
        *,
        limit: int | Omit = omit,
        pagination_key: str | Omit = omit,
        pagination_key_version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ChatAgentListResponse:
        """
        List all chat agents

        Args:
          limit: A limit on the number of objects to be returned. Limit can range between 1 and
              1000, and the default is 1000.

          pagination_key: The pagination key to continue fetching the next page of agents. Pagination key
              is represented by a agent id, pagination key and version pair is exclusive (not
              included in the fetched page). If not set, will start from the beginning.

          pagination_key_version: Specifies the version of the agent associated with the pagination_key. When
              paginating, both the pagination_key and its version must be provided to ensure
              consistent ordering and to fetch the next page correctly.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/list-chat-agents",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "limit": limit,
                        "pagination_key": pagination_key,
                        "pagination_key_version": pagination_key_version,
                    },
                    chat_agent_list_params.ChatAgentListParams,
                ),
            ),
            cast_to=ChatAgentListResponse,
        )

    async def delete(
        self,
        agent_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete an existing chat agent

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/delete-chat-agent/{agent_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def get_versions(
        self,
        agent_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ChatAgentGetVersionsResponse:
        """
        Get all versions of a chat agent

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        return await self._get(
            f"/get-chat-agent-versions/{agent_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ChatAgentGetVersionsResponse,
        )

    async def publish(
        self,
        agent_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Publish the latest version of the chat agent and create a new draft chat agent
        with newer version.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._post(
            f"/publish-chat-agent/{agent_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class ChatAgentResourceWithRawResponse:
    def __init__(self, chat_agent: ChatAgentResource) -> None:
        self._chat_agent = chat_agent

        self.create = to_raw_response_wrapper(
            chat_agent.create,
        )
        self.retrieve = to_raw_response_wrapper(
            chat_agent.retrieve,
        )
        self.update = to_raw_response_wrapper(
            chat_agent.update,
        )
        self.list = to_raw_response_wrapper(
            chat_agent.list,
        )
        self.delete = to_raw_response_wrapper(
            chat_agent.delete,
        )
        self.get_versions = to_raw_response_wrapper(
            chat_agent.get_versions,
        )
        self.publish = to_raw_response_wrapper(
            chat_agent.publish,
        )


class AsyncChatAgentResourceWithRawResponse:
    def __init__(self, chat_agent: AsyncChatAgentResource) -> None:
        self._chat_agent = chat_agent

        self.create = async_to_raw_response_wrapper(
            chat_agent.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            chat_agent.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            chat_agent.update,
        )
        self.list = async_to_raw_response_wrapper(
            chat_agent.list,
        )
        self.delete = async_to_raw_response_wrapper(
            chat_agent.delete,
        )
        self.get_versions = async_to_raw_response_wrapper(
            chat_agent.get_versions,
        )
        self.publish = async_to_raw_response_wrapper(
            chat_agent.publish,
        )


class ChatAgentResourceWithStreamingResponse:
    def __init__(self, chat_agent: ChatAgentResource) -> None:
        self._chat_agent = chat_agent

        self.create = to_streamed_response_wrapper(
            chat_agent.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            chat_agent.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            chat_agent.update,
        )
        self.list = to_streamed_response_wrapper(
            chat_agent.list,
        )
        self.delete = to_streamed_response_wrapper(
            chat_agent.delete,
        )
        self.get_versions = to_streamed_response_wrapper(
            chat_agent.get_versions,
        )
        self.publish = to_streamed_response_wrapper(
            chat_agent.publish,
        )


class AsyncChatAgentResourceWithStreamingResponse:
    def __init__(self, chat_agent: AsyncChatAgentResource) -> None:
        self._chat_agent = chat_agent

        self.create = async_to_streamed_response_wrapper(
            chat_agent.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            chat_agent.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            chat_agent.update,
        )
        self.list = async_to_streamed_response_wrapper(
            chat_agent.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            chat_agent.delete,
        )
        self.get_versions = async_to_streamed_response_wrapper(
            chat_agent.get_versions,
        )
        self.publish = async_to_streamed_response_wrapper(
            chat_agent.publish,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

from .._types import FileTypes, SequenceNotStr

__all__ = ["VoiceCloneParams"]


class VoiceCloneParams(TypedDict, total=False):
    files: Required[SequenceNotStr[FileTypes]]
    """Audio files to use for voice cloning.

    Up to 25 files allowed. For Cartesia and MiniMax, only 1 file is supported.
    """

    voice_name: Required[str]
    """Name for the cloned voice"""

    voice_provider: Literal["elevenlabs", "cartesia", "minimax"]
    """Voice provider to use for cloning. Defaults to elevenlabs."""
